{-# LANGUAGE NamedFieldPuns, OverloadedStrings #-}
module Data.Metadata (
      Authors(..)
    , Book(..)
    , Domains(..)
    , FromBook(..)
    , HasAuthors(..)
    , HasDomains(..)
    , InFile(..)
    , TXMText
    , Unique(..)
    , groupBy
    , list
    , readTsv
    , sortBy
    , tsvFile
    , tsvLines
  ) where

import Data.ByteString.Lazy as ByteString (putStr, readFile, writeFile)
import Data.ByteString.Char8 as StrictByteString (pack)
import Data.Csv
  ( DecodeOptions(..), EncodeOptions(..), FromRecord, HasHeader(..)
  , ToNamedRecord, ToRecord, decodeWith, defaultEncodeOptions, encodeByNameWith
  , encodeWith, header )
import Data.Foldable as Foldable (toList)
import Data.List (sortOn)
import Data.Map.Strict as Map (alter, empty, toList)
import Data.Metadata.Projector
  (FromBook(..), HasAuthors(..), HasDomains(..), InFile(..), TXMText, Unique(..))
import Data.Metadata.Types (Authors(..), Book(..), Domains(..))
import Data.Text as Text (Text, intercalate, unpack)
import Data.Vector as Vector (Vector)

list :: [Text] -> String
list ts = Text.unpack $ ":" <> intercalate ":" ts <> ":"

readTsv :: FromRecord a => FilePath -> IO (Either String (Vector a))
readTsv source = decodeWith fromTsv HasHeader <$> ByteString.readFile source
  where
    fromTsv = DecodeOptions {decDelimiter = fromIntegral (fromEnum '\t')}

{-
toTsv :: (Foldable t, ToNamedRecord a) => Maybe [String] -> t a -> ByteString
toTsv fields content =
  encodeByNameWith tsv (header bsFields)
  $ zipWith namedField bsFields (Foldable.toList content)
  where
    bsFields = StrictByteString.pack <$> fields
-}

toTsv :: EncodeOptions
toTsv = defaultEncodeOptions
        { encDelimiter = fromIntegral (fromEnum '\t')
        , encUseCrLf = False }

tsvFile :: ToNamedRecord a => FilePath -> [String] -> [a] -> IO ()
tsvFile target fields =
  ByteString.writeFile target
  . encodeByNameWith toTsv (header $ StrictByteString.pack <$> fields)

tsvLines :: ToRecord a => [a] -> IO ()
tsvLines = ByteString.putStr . encodeWith toTsv

{-
writeTsv :: (Foldable t, ToRecord a) => [String] -> t a -> FilePath -> IO ()
writeTsv fields content target =
  ByteString.writeFile target
  . encodeWith tsv (header bsFields)
  $ zipWith namedField bsFields (Foldable.toList content)
  where
    bsFields = StrictByteString.pack <$> fields
    tsv = defaultEncodeOptions
            { encDelimiter = fromIntegral (fromEnum '\t')
            , encUseCrLf = False }
-}


sortBy :: (Foldable t, Ord k) => (a -> k) -> t a -> [a]
sortBy field = sortOn field . Foldable.toList

groupBy :: (Foldable t, Ord k) => (v -> k) -> t v -> [(k, [v])]
groupBy field = Map.toList . foldr group Map.empty
  where
    group article = Map.alter (Just . maybe [article] (article:)) (field article)
