{- LANGUAGE DeriveGeneric, OverloadedStrings #-}
module Data.Metadata.Article
  ( Article(..) ) where

import Data.Csv (FromRecord(..), ToRecord(..))
import Data.Metadata
  ( Authors, Book, Domains
  , FromBook(..), HasAuthors(..), HasDomains(..), InFile(..), Unique(..) )
import Data.Text (Text, unpack)
import GHC.Generics (Generic)
import Text.Printf (printf)

data Article = Article
  { articleBook :: Book
  , tome :: Int
  , name :: Text
  , headWord :: Text
  , rank :: Int 
  , page :: Int 
  , articleAuthors :: Authors
  , articleDomains :: Domains } deriving (Generic, Show)

instance FromRecord Article
instance ToRecord Article
--instance ToNamedRecord Article

instance Unique Article where
  uid (Article {articleBook, tome, name}) =
    printf "%s_%d_%s" (show articleBook) tome (unpack name)

instance FromBook Article where
  book = articleBook

instance HasAuthors Article where
  authors_ = articleAuthors

instance HasDomains Article where
  domains_ = articleDomains

instance InFile Article where
  relativePath (Article {articleBook, tome, name}) =
    printf "%s/T%d/%s" (show articleBook) tome (unpack name)
