module Data.Metadata.Paragraph
  ( Paragraph(..) ) where

import Data.Csv (FromRecord(..), ToRecord(..))
import Data.Metadata
  ( Authors, Book(..), Domains
  , FromBook(..), HasAuthors(..), HasDomains(..), InFile(..), Unique(..) )
import GHC.Generics (Generic)
import Text.Printf (printf)

data Paragraph = Paragraph
  { tome :: Int
  , name :: String
  , headWord :: String
  , paragraphAuthors :: Authors
  , paragraphDomains :: Domains
  , paragraphId :: Int
  , content :: String
  , nbPlaceSpacy :: Int
  , nbPersonSpacy :: Int } deriving (Generic, Show)

instance FromRecord Paragraph
instance ToRecord Paragraph

instance Unique Paragraph where
  uid (Paragraph {tome, name, paragraphId}) =
    printf "EDdA_%d_%s_%d" tome name paragraphId

instance FromBook Paragraph where
  book _ = EDdA

instance HasAuthors Paragraph where
  authors_ = paragraphAuthors

instance HasDomains Paragraph where
  domains_ = paragraphDomains

instance InFile Paragraph where
  relativePath (Paragraph {tome, name, paragraphId}) =
    printf "T%d/article%s-%d" tome name paragraphId
