{-# LANGUAGE OverloadedStrings #-}
import Control.Applicative ((<|>), many, optional)
import Data.Attoparsec.Text as Atto
  ( Parser, char, choice, inClass, letter, many1, parseOnly, sepBy, space, string
  , takeWhile )
import Data.ByteString.Lazy as BS (getContents)
import Data.Char (isUpper, toLower, toUpper)
import Data.Csv (HasHeader(..), decode)
import Data.Text as Text (Text, pack)
import Data.Text.IO as Text (getContents, readFile)
import System.Script (syntax, warn)
import System.Environment (getArgs)
import System.Exit (die)
import System.FilePath ((</>))
import Text.Printf (printf)

eddaFirstWord :: String -> Parser String
eddaFirstWord headWord =
  diderotStar *> many space *> head *> meta *> optional qualification *> many1 letter
  where
    diderotStar = optional (char '*')
    pulp = Atto.takeWhile (inClass ",. \t\n\r")
    head = string (pack headWord) <|> Atto.takeWhile isUpper --choice [Atto.takeWhile isUpper, string (pack headWord)]
    meta = pulp `sepBy` (abbreviation <|> parenthesis)
    --meta = pulp `sepBy` (choice [abbreviation, parenthesis])
    --head = string (pack headWord) <* pulp `sepBy` (choice [abbreviation, parenthesis])
    abbreviation = pack <$> many1 letter <* char '.'
    --head = string (pack headWord) <* pulp `sepBy` (choice [gram, parenthesis])
    gram = choice [noun, adj] <* char '.'
    noun = string "s." <* optional space <* choice [char 'm', char 'f']
    adj = string "adj"

parenthesis :: Parser Text
parenthesis = char '(' *> Atto.takeWhile (/= ')') <* char ')'

qualification :: Parser Text
qualification =
  qualifier <* optional (char 'e') <* optional (char 's') <* many space
  where
    qualifier = choice $ string . pack <$> (vocabulary ++ (ucFirst <$> vocabulary))
    vocabulary = ["petit", "grand", "vaste", "belle", "fameux", "fameuse"]
    ucFirst (c:cs) = toUpper c : cs

lgeFirstWord :: String -> Parser String
lgeFirstWord headWord = head *> pulp *> meta *> optional qualification *> pulp *> many1 letter
  where
    head = string (pack headWord)
    pulp = Atto.takeWhile (inClass ",. \t\n\r")
    meta = many (parenthesis *> pulp)

extractFirstWord :: FilePath -> (String -> Parser String)
  -> (String, Int, Int, String, String) -> IO ()
  -- -> (Int, Int, String, String, String, String) -> IO ()
extractFirstWord rootDirectory parser (_,tome, name, headWord, _) =
--extractFirstWord rootDirectory parser (tome, name, headWord, _, _, _) =
  Text.readFile (rootDirectory </> (printf "T%d/ByRank/%d.txt" tome name))
  --Text.readFile (rootDirectory </> (printf "T%d/article%d.txt" tome name))
  >>= pure . parseOnly (parser headWord)
  >>= either
        (\_ -> warn $ printf "%d\t%d" tome name)
        (\word -> putStrLn $ printf "%d\t%d\t%s" tome name word)

run :: [String] -> IO ()
run [book, rootDirectory]
  | normalizedBook == "edda" = apply eddaFirstWord
  | normalizedBook == "lge" = apply lgeFirstWord
  where
    normalizedBook = toLower <$> book
    apply parser =
      BS.getContents
      >>= either die (mapM_ $ extractFirstWord rootDirectory parser) . decode HasHeader

run _ = syntax "BOOK[either EDdA or LGE] SOURCE_DIRECTORY path to the root directory containing the articles (which CSV metadata are read from stdin)"

main :: IO ()
main = getArgs >>= run
