#!/usr/bin/env python3

import os
import os.path
import stanza
from stanza.utils.conll import CoNLL
import sys

def oneLine(sentence):
    return sentence.text.replace('\n', ' ').replace('\t', ' ')

class Annotator:
    def __init__(self, source, target):
        self.source = source
        self.target = target
        self.paragraphSerial = -1
        self.sentenceSerial = -1
        self.model = stanza.Pipeline(
                lang='fr',
                processors='tokenize,pos,lemma,depparse')

    def run(self):
        for name in os.listdir(self.source):
            if os.path.isdir(f'{self.source}/{name}'):
                self.annotate_dir(name)

    def annotate_dir(self, directory):
        source_path = f'{self.source}/{directory}'
        os.makedirs(f'{self.target}/{directory}', exist_ok=True)
        for name in os.listdir(source_path):
            path = f'{source_path}/{name}'
            relpath = f'{directory}/{name}'
            if os.path.isdir(path):
                self.annotate_dir(relpath)
            elif os.path.isfile(path):
                self.annotate_file(relpath)

    def annotate_file(self, file):
        outputFile = f'{self.target}/{file[:-4]}.conllu'
        if not os.path.isfile(outputFile):
            with open(f'{self.source}/{file}', 'r') as source:
                document = source.read()
                print("trying " + outputFile)
                try:
                    with open(outputFile, 'w') as target:
                        for paragraph in document.split('\n\n'):
    #                        print(f'# newpar id = {self.newpar()}', file=target)
                            self.annotate_paragraph(paragraph, target)
                except Exception:
                    os.unlink(outputFile)
                    sys.exit(1)
                    

    def annotate_paragraph(self, paragraph, target):
        parsed = self.model(paragraph)
        if len(parsed.sentences) > 0:
            print(f'# newpar id = {self.newpar()}', file=target)
            for sentence in parsed.sentences:
                sentence.add_comment(f'# sent_id = {self.newsent()}')
                sentence.add_comment(f'# text = {oneLine(sentence)}')
            print(CoNLL.doc2conll_text(parsed), file=target, end='')

#    def annotate_paragraph(self, paragraph, target):
#        parsed = self.model(paragraph)
#        for sentence in parsed.sentences:
#            sentence.add_comment(f'# sent_id = {self.newsent()}')
#            sentence.add_comment(f'# text = {oneLine(sentence)}')
#        print(CoNLL.doc2conll_text(parsed), file=target, end='')

    def newpar(self):
        self.paragraphSerial += 1
        return self.paragraphSerial

    def newsent(self):
        self.sentenceSerial += 1
        return self.sentenceSerial

if __name__ == '__main__':
    Annotator(sys.argv[1], sys.argv[2]).run()
