#!/usr/bin/env python3

from lxml import etree
import os
import os.path
import stanza
from stanza.utils.conll import CoNLL
import sys

def oneLine(sentence):
    return sentence.text.replace('\n', ' ').replace('\t', ' ')

class Annotator:
    def __init__(self, source, target, xpath):
        self.source = source
        self.target = target
        self.xpath = xpath
        self.paragraphSerial = -1
        self.sentenceSerial = -1
        self.model = stanza.Pipeline(
                lang='fr',
                processors='tokenize,pos,lemma,depparse')

    def run(self):
        for name in os.listdir(self.source):
            if os.path.isdir(f'{self.source}/{name}'):
                self.annotate_dir(name)

    def annotate_dir(self, directory):
        source_path = f'{self.source}/{directory}'
        os.makedirs(f'{self.target}/{directory}', exist_ok=True)
        for name in os.listdir(source_path):
            path = f'{source_path}/{name}'
            relpath = f'{directory}/{name}'
            if os.path.isdir(path):
                self.annotate_dir(relpath)
            elif os.path.isfile(path):
                self.annotate_file(relpath)

    def annotate_file(self, file):
        outputFile = f'{self.target}/{file[:-4]}.conllu'
        if not os.path.isfile(outputFile):
            document = etree.parse(f'{self.source}/{file}')
            with open(outputFile, 'w') as target:
                for paragraph in document.xpath(self.xpath):
                    self.annotate_paragraph(paragraph, target)

    def annotate_paragraph(self, paragraph, target):
        parsed = self.model(" ".join(paragraph.itertext()))
        if len(parsed.sentences) > 0:
            print(f'# newpar id = {self.newpar()}', file=target)
            for sentence in parsed.sentences:
                sentence.add_comment(f'# sent_id = {self.newsent()}')
                sentence.add_comment(f'# text = {oneLine(sentence)}')
            print(CoNLL.doc2conll_text(parsed), file=target, end='')

    def newpar(self):
        self.paragraphSerial += 1
        return self.paragraphSerial

    def newsent(self):
        self.sentenceSerial += 1
        return self.sentenceSerial

if __name__ == '__main__':
    paragraphXPath = '/TEI/text/body/div1/p'
    if len(sys.argv) > 3:
        paragraphXPath = sys.argv[3]
    Annotator(sys.argv[1], sys.argv[2], paragraphXPath).run()
